@@ -520,6 +520,16 @@ void textOut(uint16_t x, uint16_t y, uint8_t paletteIndex, const char *textPtr)
 	}
 }
 
+void textOutBorder(uint16_t x, uint16_t y, uint8_t paletteIndex, uint8_t borderPaletteIndex, const char *textPtr)
+{
+	textOut(x,   y-1, borderPaletteIndex, textPtr); // top
+	textOut(x+1, y,   borderPaletteIndex, textPtr); // right
+	textOut(x,   y+1, borderPaletteIndex, textPtr); // bottom
+	textOut(x-1, y,   borderPaletteIndex, textPtr); // left
+
+	textOut(x, y, paletteIndex, textPtr);
+}
+
 // fixed width
 void textOutFixed(uint16_t x, uint16_t y, uint8_t fgPaltete, uint8_t bgPalette, const char *textPtr)
 {
@@ -724,22 +734,22 @@ void fillRect(uint16_t xPos, uint16_t yPos, uint16_t w, uint16_t h, uint8_t pale
 
 void blit32(uint16_t xPos, uint16_t yPos, const uint32_t* srcPtr, uint16_t w, uint16_t h)
 {
-    uint32_t* dstPtr;
+	uint32_t* dstPtr;
 
-    assert(srcPtr != NULL && xPos < SCREEN_W && yPos < SCREEN_H && (xPos + w) <= SCREEN_W && (yPos + h) <= SCREEN_H);
+	assert(srcPtr != NULL && xPos < SCREEN_W && yPos < SCREEN_H && (xPos + w) <= SCREEN_W && (yPos + h) <= SCREEN_H);
 
-    dstPtr = &video.frameBuffer[(yPos * SCREEN_W) + xPos];
-    for (uint32_t y = 0; y < h; y++)
-    {
-        for (uint32_t x = 0; x < w; x++)
-        {
-            if (srcPtr[x] != 0x00FF00)
-                dstPtr[x] = srcPtr[x] | 0xFF000000; // most significant 8 bits = palette number. 0xFF because no true palette
-        }
+	dstPtr = &video.frameBuffer[(yPos * SCREEN_W) + xPos];
+	for (uint32_t y = 0; y < h; y++)
+	{
+		for (uint32_t x = 0; x < w; x++)
+		{
+			if (srcPtr[x] != 0x00FF00)
+				dstPtr[x] = srcPtr[x] | 0xFF000000; // most significant 8 bits = palette number. 0xFF because no true palette
+		}
 
-        srcPtr += w;
-        dstPtr += SCREEN_W;
-    }
+		srcPtr += w;
+		dstPtr += SCREEN_W;
+	}
 }
 
 void blit(uint16_t xPos, uint16_t yPos, const uint8_t *srcPtr, uint16_t w, uint16_t h)
