@@ -111,6 +111,8 @@ void resetChannels(void)
 		ch->oldPan = 128;
 		ch->outPan = 128;
 		ch->finalPan = 128;
+
+		ch->stOff = !editor.chnMode[i]; // set channel mute flag from global mute flag
 	}
 
 	if (audioWasntLocked)
@@ -148,79 +150,31 @@ void tuneSample(sampleTyp *s, uint32_t midCFreq)
 	}
 }
 
-bool setPatternLen(uint16_t nr, int16_t len)
+void setPatternLen(uint16_t nr, int16_t len)
 {
 	bool audioWasntLocked;
-	tonTyp *newPtr;
 
 	assert(nr < MAX_PATTERNS);
 
-	len = CLAMP(len, 1, MAX_PATT_LEN);
-	if (len == pattLens[nr])
-		return true;
+	if ((len < 1 || len > MAX_PATT_LEN) || len == pattLens[nr])
+		return;
 
 	audioWasntLocked = !audio.locked;
 	if (audioWasntLocked)
 		lockAudio();
 
-	if (patt[nr] == NULL)
-	{
-		pattLens[nr] = len;
+	pattLens[nr] = len;
 
-		if (editor.editPattern == song.pattNr)
-			song.pattLen = pattLens[nr];
-
-		if (song.pattPos >= pattLens[nr])
-			song.pattPos = pattLens[nr] - 1;
-
-		editor.pattPos = song.pattPos;
-
-		checkMarkLimits();
-
-		if (audioWasntLocked)
-			unlockAudio();
-
-		editor.ui.updatePatternEditor = true;
-		editor.ui.updatePosSections = true;
-
-		return true;
-	}
-
-	newPtr = (tonTyp *)realloc(patt[nr], len * TRACK_WIDTH);
-	if (newPtr == NULL)
-	{
-		okBox(0, "Status message", "Not enough memory!");
-
-		if (audioWasntLocked)
-			unlockAudio();
-
-		return false;
-	}
-
-	patt[nr] = newPtr;
-
-	// if we enlarged the pattern length, wipe the new data
-	if (len >= pattLens[nr])
-	{
-		if (len > pattLens[nr])
-			memset(&patt[nr][pattLens[nr] * MAX_VOICES], 0, (len - pattLens[nr]) * TRACK_WIDTH);
-
-		pattLens[nr] = len;
-	}
-	else
-	{
-		pattLens[nr] = len;
+	if (patt[nr] != NULL)
 		killPatternIfUnused(nr);
+
+	song.pattLen = pattLens[nr];
+	if (song.pattPos >= song.pattLen)
+	{
+		song.pattPos = song.pattLen - 1;
+		editor.pattPos = song.pattPos;
 	}
 
-	if (editor.editPattern == song.pattNr)
-		song.pattLen = pattLens[nr];
-
-	if (song.pattPos >= pattLens[nr])
-		song.pattPos = pattLens[nr] - 1;
-
-	editor.pattPos = song.pattPos;
-
 	checkMarkLimits();
 
 	if (audioWasntLocked)
@@ -228,8 +182,6 @@ bool setPatternLen(uint16_t nr, int16_t len)
 
 	editor.ui.updatePatternEditor = true;
 	editor.ui.updatePosSections = true;
-
-	return true;
 }
 
 int16_t getUsedSamples(int16_t nr)
@@ -470,7 +422,7 @@ static void startTone(uint8_t ton, uint8_t effTyp, uint8_t eff, stmTyp *ch)
 	if (ton > 96) // non-FT2 security (should never happen because I clamp in the patt loaders now)
 		ton = 96;
 
-	smp = ins->ta[ton-1] & 0x0F;
+	smp = ins->ta[ton-1] & 0xF;
 	ch->sampleNr = smp;
 
 	s = &ins->samp[smp];
@@ -1256,9 +1208,9 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 	uint8_t envPos;
 	int16_t autoVibVal, panTmp;
 	uint16_t autoVibAmp, tmpPeriod, envVal;
-	int32_t vol, tmp32;
+	int32_t tmp32;
+	uint32_t vol;
 	instrTyp *ins;
-	double dVol;
 
 	ins = ch->instrSeg;
 
@@ -1305,7 +1257,7 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 						{
 							envPos = ins->envVRepS;
 							ch->envVCnt = ins->envVP[envPos][0];
-							ch->envVAmp = (ins->envVP[envPos][1] & 0xFF) << 8;
+							ch->envVAmp = ins->envVP[envPos][1] << 8;
 						}
 					}
 
@@ -1332,7 +1284,7 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 						ch->envVIPValue = 0;
 						if (ins->envVP[envPos][0] > ins->envVP[envPos-1][0])
 						{
-							ch->envVIPValue = ((ins->envVP[envPos][1] - ins->envVP[envPos-1][1]) & 0xFF) << 8;
+							ch->envVIPValue = (ins->envVP[envPos][1] - ins->envVP[envPos-1][1]) << 8;
 							ch->envVIPValue /= (ins->envVP[envPos][0] - ins->envVP[envPos-1][0]);
 
 							envVal = ch->envVAmp;
@@ -1351,10 +1303,10 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 				ch->envVAmp += ch->envVIPValue;
 
 				envVal = ch->envVAmp;
-				if ((envVal >> 8) > 0x40)
+				if (envVal > 64*256)
 				{
-					if ((envVal >> 8) > (0x40+0xC0)/2)
-						envVal = 16384;
+					if (envVal > 128*256)
+						envVal = 64*256;
 					else
 						envVal = 0;
 
@@ -1362,41 +1314,36 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 				}
 			}
 
-			/* old integer method with low precision (FT2 way)
+			/* original FT2 method with lower precision (finalVol = 0..256)
 			envVal >>= 8;
 			ch->finalVol = (song.globVol * (((envVal * ch->outVol) * ch->fadeOutAmp) >> (16 + 2))) >> 7;
 			*/
 
-			/* calculate with four times more precision (+ rounding).
-			** also, env. range is now 0..16384 instead of being shifted to 0..64. */
+			/* calculate with four times more precision (finalVol = 0..2048)
+			** Also, vol envelope range is now 0..16384 instead of being shifted to 0..64
+			*/
 
-			dVol = song.globVol * ch->outVol * ch->fadeOutAmp;
-			dVol *= envVal; // we need a float mul because it would overflow 32-bit integer
-			dVol *= 1.0 / ((64.0 * 64.0 * 32768.0 * 16384.0) / 2048.0); // 0..2048 (real FT2 is 0..256)
+			uint32_t vol1 = song.globVol * ch->outVol * ch->fadeOutAmp; // 0..64 * 0..64 * 0..32768 = 0..134217728
+			uint32_t vol2 = envVal << 2; // 0..16384 * 2^2 = 0..65536
 
-			vol = (int32_t)(dVol + 0.5);
-			if (vol > 2048)
-				vol = 2048;
+			vol = ((uint64_t)vol1 * vol2) >> 32; // 0..2048
 
-			ch->finalVol = (uint16_t)vol;
 			ch->status |= IS_Vol;
 		}
 		else
 		{
-			/* old integer method with low precision (FT2 way)
+			/* original FT2 method with lower precision (finalVol = 0..256)
 			ch->finalVol = (song.globVol * (((ch->outVol << 4) * ch->fadeOutAmp) >> 16)) >> 7;
 			*/
 
-			// calculate with four times more precision (+ rounding)
-			dVol = song.globVol * ch->outVol * ch->fadeOutAmp;
-			dVol *= 1.0 / ((64.0 * 64.0 * 32768.0) / 2048.0); // 0..2048 (real FT2 is 0..256)
-
-			vol = (int32_t)(dVol + 0.5);
-			if (vol > 2048)
-				vol = 2048;
-
-			ch->finalVol = (uint16_t)vol;
+			// calculate with four times more precision (finalVol = 0..2048)
+			vol = (song.globVol * ch->outVol * ch->fadeOutAmp) >> 16; // 0..64 * 0..64 * 0..32768 = 0..2048
 		}
+
+		if (vol > 2047)
+			vol = 2047; // range is now 0..2047 to prevent MUL overflow when voice volume is calculated
+
+		ch->finalVol = (uint16_t)vol;
 	}
 	else
 	{
@@ -1413,7 +1360,7 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 
 		if (++ch->envPCnt == ins->envPP[envPos][0])
 		{
-			ch->envPAmp = (ins->envPP[envPos][1] & 0xFF) << 8;
+			ch->envPAmp = ins->envPP[envPos][1] << 8;
 
 			envPos++;
 			if (ins->envPTyp & 4)
@@ -1427,7 +1374,7 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 						envPos = ins->envPRepS;
 
 						ch->envPCnt = ins->envPP[envPos][0];
-						ch->envPAmp = (ins->envPP[envPos][1] & 0xFF) << 8;
+						ch->envPAmp = ins->envPP[envPos][1] << 8;
 					}
 				}
 
@@ -1454,7 +1401,7 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 					ch->envPIPValue = 0;
 					if (ins->envPP[envPos][0] > ins->envPP[envPos-1][0])
 					{
-						ch->envPIPValue = ((ins->envPP[envPos][1] - ins->envPP[envPos-1][1]) & 0xFF) << 8;
+						ch->envPIPValue = (ins->envPP[envPos][1] - ins->envPP[envPos-1][1]) << 8;
 						ch->envPIPValue /= (ins->envPP[envPos][0] - ins->envPP[envPos-1][0]);
 
 						envVal = ch->envPAmp;
@@ -1473,10 +1420,10 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 			ch->envPAmp += ch->envPIPValue;
 
 			envVal = ch->envPAmp;
-			if ((envVal >> 8) > 0x40)
+			if (envVal > 64*256)
 			{
-				if ((envVal >> 8) > (0x40+0xC0)/2)
-					envVal = 16384;
+				if (envVal > 128*256)
+					envVal = 64*256;
 				else
 					envVal = 0;
 
@@ -1489,9 +1436,9 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 			panTmp = 0 - panTmp;
 		panTmp += 128;
 
-		envVal -= (32 * 256);
+		envVal -= 32*256;
 
-		ch->finalPan = ch->outPan + ((((int16_t)envVal * (panTmp << 3)) >> 16) & 0xFF);
+		ch->finalPan = ch->outPan + (uint8_t)(((int16_t)envVal * panTmp) >> 13);
 		ch->status |= IS_Pan;
 	}
 	else
@@ -1500,7 +1447,11 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 	}
 
 	// *** AUTO VIBRATO ***
+#ifdef HAS_MIDI
 	if (ch->midiVibDepth > 0 || ins->vibDepth > 0)
+#else
+	if (ins->vibDepth > 0)
+#endif
 	{
 		if (ch->eVibSweep > 0)
 		{
@@ -1522,12 +1473,13 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 			autoVibAmp = ch->eVibAmp;
 		}
 
+#ifdef HAS_MIDI
 		// non-FT2 hack to make modulation wheel work when auto vibrato rate is zero
 		if (ch->midiVibDepth > 0 && ins->vibRate == 0)
 			ins->vibRate = 0x20;
 
 		autoVibAmp += ch->midiVibDepth;
-
+#endif
 		ch->eVibPos += ins->vibRate;
 
 		     if (ins->vibTyp == 1) autoVibVal = (ch->eVibPos > 127) ? 64 : -64; // square
@@ -1548,11 +1500,14 @@ static void fixaEnvelopeVibrato(stmTyp *ch)
 	else
 	{
 		ch->finalPeriod = ch->outPeriod;
+
+#ifdef HAS_MIDI
 		if (midi.enable)
 		{
 			ch->finalPeriod -= ch->midiPitch;
 			ch->status |= IS_Period;
 		}
+#endif
 	}
 }
 
@@ -2203,7 +2158,7 @@ void resetMusic(void)
 	if (audioWasntLocked)
 		unlockAudio();
 
-	setPos(0, 0);
+	setPos(0, 0, false);
 
 	if (!songPlaying)
 	{
@@ -2212,7 +2167,7 @@ void resetMusic(void)
 	}
 }
 
-void setPos(int16_t songPos, int16_t pattPos)
+void setPos(int16_t songPos, int16_t pattPos, bool resetTimer)
 {
 	bool audioWasntLocked = !audio.locked;
 	if (audioWasntLocked)
@@ -2255,7 +2210,8 @@ void setPos(int16_t songPos, int16_t pattPos)
 		}
 	}
 
-	song.timer = 1;
+	if (resetTimer)
+		song.timer = 1;
 
 	if (audioWasntLocked)
 		unlockAudio();
@@ -2628,7 +2584,7 @@ void updateChanNums(void)
 		}
 	}
 
-	editor.ui.pattChanScrollShown = (song.antChn > getMaxVisibleChannels());
+	editor.ui.pattChanScrollShown = song.antChn > getMaxVisibleChannels();
 
 	if (editor.ui.patternEditorShown)
 	{
@@ -2836,7 +2792,7 @@ bool setupReplayer(void)
 	song.initialTempo = song.tempo;
 
 	setFrqTab(true);
-	setPos(0, 0);
+	setPos(0, 0, true);
 
 	if (!allocateInstr(0))
 	{
@@ -2876,9 +2832,9 @@ void startPlaying(int8_t mode, int16_t row)
 	assert(mode != PLAYMODE_IDLE && mode != PLAYMODE_EDIT);
 
 	if (mode == PLAYMODE_PATT || mode == PLAYMODE_RECPATT)
-		setPos(-1, row);
+		setPos(-1, row, true);
 	else
-		setPos(editor.songPos, row);
+		setPos(editor.songPos, row, true);
 
 	playMode = mode;
 	songPlaying = true;
@@ -2926,8 +2882,10 @@ void stopPlaying(void)
 	if (songWasPlaying)
 		editor.pattPos = song.pattPos;
 
+#ifdef HAS_MIDI
 	midi.currMIDIVibDepth = 0;
 	midi.currMIDIPitch = 0;
+#endif
 
 	memset(editor.keyOnTab, 0, sizeof (editor.keyOnTab));
 
@@ -3181,12 +3139,15 @@ void stopVoices(void)
 
 void decSongPos(void)
 {
+	if (song.songPos == 0)
+		return;
+
 	bool audioWasntLocked = !audio.locked;
 	if (audioWasntLocked)
 		lockAudio();
 
 	if (song.songPos > 0)
-		setPos(song.songPos - 1, 0);
+		setPos(song.songPos - 1, 0, true);
 
 	if (audioWasntLocked)
 		unlockAudio();
@@ -3194,12 +3155,15 @@ void decSongPos(void)
 
 void incSongPos(void)
 {
+	if (song.songPos == song.len-1)
+		return;
+
 	bool audioWasntLocked = !audio.locked;
 	if (audioWasntLocked)
 		lockAudio();
 
 	if (song.songPos < song.len-1)
-		setPos(song.songPos + 1, 0);
+		setPos(song.songPos + 1, 0, true);
 
 	if (audioWasntLocked)
 		unlockAudio();
